module acm2108_ddr3_uart(
    clk,
    reset_n,
    //LED
    led,
    //UART
    uart_rx,
    uart_tx,
    //ACM2108
    key_in,   //л
    AD0,
    AD1,
    AD0_CLK,
    AD1_CLK,
    DA0_Data,
    DA1_Data,
    DA0_Clk,
    DA1_Clk, 
    //DDR3 Interface
    // Inouts
    ddr3_dq,
    ddr3_dqs_n,
    ddr3_dqs_p,
    // Outputs
    ddr3_addr,
    ddr3_ba,
    ddr3_ras_n,
    ddr3_cas_n,
    ddr3_we_n,
    ddr3_reset_n,
    ddr3_ck_p,
    ddr3_ck_n,
    ddr3_cke,
    ddr3_cs_n,
    ddr3_dm,
    ddr3_odt
    );
    input        clk;
    input        reset_n;
    
    //л
    input        key_in;
    
    //LED
    output [1:0] led;
    //UART
    input        uart_rx;
    output       uart_tx;
    //ACM2108
    input        [7:0]AD0; 
	input        [7:0]AD1;
	output       AD0_CLK;
	output       AD1_CLK;
	output       [7:0]DA0_Data; 
	output       [7:0]DA1_Data;
	output       DA0_Clk;
	output       DA1_Clk;

    //DDR3 Interface
    // Inouts
    inout  [15:0]ddr3_dq;
    inout  [1:0] ddr3_dqs_n;
    inout  [1:0] ddr3_dqs_p;
    // Outputs
    output [13:0]ddr3_addr;
    output [2:0] ddr3_ba;
    output       ddr3_ras_n;
    output       ddr3_cas_n;
    output       ddr3_we_n;
    output       ddr3_reset_n;
    output [0:0] ddr3_ck_p;
    output [0:0] ddr3_ck_n;
    output [0:0] ddr3_cke;
    output [0:0] ddr3_cs_n;
    output [1:0] ddr3_dm;
    output [0:0] ddr3_odt;
    
    wire clk_50M;
    wire clk_200M; //200MʱӹDDR3ʹ
    wire pll_locked; //PLLź
    wire g_reset;
    
    wire [2:0] uart_baud_set; //
    wire [7:0] uart_rx_data; //Ҫ͵
    wire uart_rx_done; //ֽڷź
    
    wire [7:0]cmd_addr; //ĵַΣ00 ʼ01òͨţ03ò
    wire [31:0]cmd_data; //ݶΣҪõĲȻ߲ͨ
    wire cmdvalid;  //Чź
    
    wire [31:0]set_sample_speed;   //ADCĲ
    wire [1:0]adc_ch_sel; //ADCͨѡܹͨ
    wire [15:0]ad_out;   //ADCź
    wire ad_out_valid;   //ADCЧź
    wire start_sample;  //ʼݴź
    wire [15:0]set_sample_num; //Ҫź
    
    wire ad_sample_en; 
    wire adc_data_en;   //ADC洢ʹź
    
    wire uart_send_en;
    wire [7:0]uart_tx_data;
    wire uart_tx_done;
  
    wire ddr3_init_done;   //DDR3ʼɱ־ź
    
    wire wrfifo_clr;
    wire wrfifo_full;
    wire wrfifo_wren;
    
    wire rdfifo_clr;
    wire rdfifo_empty;
    wire rdfifo_rden;
    wire [15:0]rdfifo_dout;
    wire [11:0]ad_out_s;   
    wire ddr3_rst_n;
     
    assign g_reset = ~pll_locked;
//    assign ad_clk1 = ~clk_50M;
//    assign ad_clk2 = ~clk_50M;
    
    assign ddr3_rst_n = pll_locked;
     //led[0] ໷źΪߣ˵໷ʱ
    //led[1] DDR3ʼɱʶźţΪߣ˵DDR3Ѿɳʼ
    assign led = {ddr3_init_done,pll_locked};
    
    wire clk125m;
    wire AD_Clk;
    wire clk50m;
    
    //PLLģ飺200MʱӹDDR3ʹãģʹ50Mʱ
  pll pll
   (
    // Clock out ports
    .clk_out1(clk_50M),     // output clk_out1
    .clk_out2(clk_200M),     // output clk_out2
    .clk_out3(clk125m),
    .clk_out4(AD_Clk),
    // Status and control signals
    .resetn(reset_n), // input resetn
    .locked(pll_locked),       // output locked
   // Clock in ports
    .clk_in1(clk));      // input clk_in1
    
    //ڽģ飺
     uart_byte_rx uart_byte_rx(
        .clk       (clk_50M      ),
        .reset     (g_reset      ),
        .baud_set  (uart_baud_set),
        .uart_rx   (uart_rx      ),
        .data_byte (uart_rx_data ),
        .rx_done   (uart_rx_done )
    );
    
    //ģ飺ڽյתɶӦź
    uart_cmd uart_cmd_inst(
        .Clk      (clk_50M   ),
        .Reset_n  (~g_reset     ),
        .rx_data  (uart_rx_data ),
        .rx_done  (uart_rx_done ),
        .address  (cmd_addr     ),
        .data     (cmd_data     ),
        .cmdvalid (cmdvalid     )
    );    
    
    //תģ飺תADCصź
    uart_cmd_rx uart_cmd_rx(
        .clk(clk_50M),
        .reset(g_reset),
        .uart_baud_set(uart_baud_set),
        .adc_ch_sel(adc_ch_sel),
        .set_sample_num(set_sample_num),
        .set_sample_speed(set_sample_speed),    
        .start_sample(start_sample),
        .cmdvalid(cmdvalid),
        .cmd_addr(cmd_addr),
        .cmd_data(cmd_data)
    );
    
    //ٶģ飺ADCĲٶ
    speed_ctrl speed_ctrl(
        .clk(clk_50M),
        .reset_n(~g_reset),
        .ad_sample_en(ad_sample_en),
        .adc_data_en(adc_data_en),
        .div_set(set_sample_speed)
    );
    
    //acm21088λת16λݣλз
    ad_8bit_to_16bit(
       .clk(clk_50M),
       .ad_sample_en(ad_sample_en),
       .ch_sel(adc_ch_sel),
       .AD0(AD0),
       .AD1(AD1),
       .ad_out(ad_out),
       .ad_out_valid(ad_out_valid)
    );
    
    acm2108_test acm2108_test(
        .Clk(clk_50M),
        .key_in(key_in),
        .AD0(AD0), 
        .AD1(AD1),
        .clk125m(clk125m),
        .clk50m(clk_50M),
        .AD_Clk(AD_Clk),
        .AD0_CLK(AD0_CLK),
        .AD1_CLK(AD1_CLK),
        .DA0_Data(DA0_Data), 
        .DA1_Data(DA1_Data),
        .DA0_Clk(DA0_Clk),
        .DA1_Clk(DA1_Clk)
     );
   
   //״̬ģ
    state_ctrl state_ctrl(
        .clk            (clk_50M    ),
        .reset          (g_reset       ),
        
        .start_sample   (start_sample  ),
        .set_sample_num (set_sample_num),
        
        .ddr3_init_done (ddr3_init_done),
        .uart_tx_done   (uart_tx_done  ),
        .rdfifo_empty   (rdfifo_empty  ),
        .rdfifo_dout    (rdfifo_dout   ),
        .wrfifo_full    (wrfifo_full   ),
        .wrfifo_clr     (wrfifo_clr    ),
        .ad_sample_en   (ad_sample_en  ),
        .adc_data_en    (adc_data_en   ),
        .ad_out_valid   (ad_out_valid  ),
        .wrfifo_wren    (wrfifo_wren   ),
        .rdfifo_clr     (rdfifo_clr    ),
        .rdfifo_rden    (rdfifo_rden   ),
        .uart_send_en   (uart_send_en  ),
        .uart_tx_data   (uart_tx_data  )
    );
   
    //DDR3˫˿ģ
    ddr3_ctrl_2port #(
    .DW            (16),
    .WR_ADDR_BEGIN (0),//洢дʼλ
    .WR_ADDR_END   (32768),//洢дֹλ
    .RD_ADDR_BEGIN (0),//洢Ķʼλ
    .RD_ADDR_END   (32768)//洢Ķֹλ
  )
  ddr3_ctrl_2port(
    .ddr3_clk200m  (clk_200M   ),
    .ddr3_rst_n    (ddr3_rst_n    ),
    .ddr3_init_done(ddr3_init_done),
    .wrfifo_clr    (wrfifo_clr    ),
    .wrfifo_clk    (clk_50M    ),
    .wrfifo_wren   (ad_out_valid && adc_data_en),
    .wrfifo_din    (ad_out        ),
    .wrfifo_full   (wrfifo_full   ),
    .wrfifo_wr_cnt (              ),
    //rd_fifo Interface
    .rdfifo_clr    (rdfifo_clr    ),
    .rdfifo_clk    (clk_50M    ),
    .rdfifo_rden   (rdfifo_rden   ),
    .rdfifo_dout   (rdfifo_dout   ),
    .rdfifo_empty  (rdfifo_empty  ),
    .rdfifo_rd_cnt (              ),
    // Inouts
    .ddr3_dq       (ddr3_dq       ),
    .ddr3_dqs_n    (ddr3_dqs_n    ),
    .ddr3_dqs_p    (ddr3_dqs_p    ),
    // Outputs      
    .ddr3_addr     (ddr3_addr     ),
    .ddr3_ba       (ddr3_ba       ),
    .ddr3_ras_n    (ddr3_ras_n    ),
    .ddr3_cas_n    (ddr3_cas_n    ),
    .ddr3_we_n     (ddr3_we_n     ),
    .ddr3_reset_n  (ddr3_reset_n  ),
    .ddr3_ck_p     (ddr3_ck_p     ),
    .ddr3_ck_n     (ddr3_ck_n     ),
    .ddr3_cke      (ddr3_cke      ),
    .ddr3_cs_n     (ddr3_cs_n     ),
    .ddr3_dm       (ddr3_dm       ),
    .ddr3_odt      (ddr3_odt      )
  );
    
    //ڷģ飺ɼͨڷ
    uart_byte_tx uart_byte_tx(
        .clk       (clk_50M   ),//50Mʱ,
        .reset     (g_reset      ),//ģ鸴λ,
        .data_byte (uart_tx_data ),//8bit,
        .send_en   (uart_send_en ),//ʹ,
        .baud_set  (uart_baud_set),//ڲ
        .uart_tx   (uart_tx      ),//Rs232ź,
        .tx_done   (uart_tx_done )//һηɱ־,
    );

endmodule
